# -------------------------------------------------------------------------------------
# MKDoc::Lang::Generic
# -------------------------------------------------------------------------------------
#
#       Author : Jean-Michel Hiver (jhiver@mkdoc.com).
#    Copyright : (c) MKDoc Holdings Ltd, 2001
# 
#      Unauthorized modification, use, reuse, distribution or redistribution
#      of this module is stricly forbidden.
#
#    Description:
#
#      Language module for potentially ALL languages, attempts to be as generic
#      as possible.
#
# -------------------------------------------------------------------------------------
package MKDoc::Lang::Generic;
use Unicode::String qw /latin1 utf8/;
use Text::Unidecode;
use strict;
use 5.008_000;
use utf8;


##
# $class->new;
# ------------
#   Instanciates a new MKDoc::Lang::Generic object.
##
sub new
{
    my $class = shift;
    $class = ref $class || $class;
    my $self = bless { @_ }, $class;
    return $self;
}


##
# $self->to_latin ($string);
# --------------------------
#     $string - UTF-8 Unicode $string
#
#   Returns a romanized version of $string in the ISO-8859-1 charset.
##
sub to_latin
{
    my $self   = shift;
    my $string = shift; # u_shift
    return unidecode ($string);
}


##
# $self->to_ascii ($string);
# --------------------------
#     $string - UTF-8 Unicode $string
#
#   Returns a romanized version of $string in the US ASCII charset.
##
sub to_ascii
{
    my $self   = shift;
    my $string = shift; # u_shift
    return unidecode ($string);
}


##
# $self->ascii_keywords ($string);
# --------------------------------
#     $string - UTF-8 Unicode $string
#
#   Split the results of $self->to_ascii into a set of keywords
#   and returns it as a list.
##
sub ascii_keywords
{
    my $self = shift;
    my $value = shift;
    $value = $self->to_ascii ($value);
    
    # uppercase every char
    $value = uc $value;
    
    # turn all wierd char into nothing (accents, commas,...)
    $value =~ s/[^A-Z ]/ /gs;
    
    # remove trailing s'es
    $value =~ s/S\s+/ /gs;
    $value =~ s/S$/ /gs;
    
    my @value = split /\s+/, $value;
    my %res;
    foreach my $value (@value)
    {
	next if ($value =~ /^\s*$/);
	$res{$value} = 1;
    }
    
    return keys %res;
}


1;
