# -------------------------------------------------------------------------------------
# flo::plugin::Account::Remove
# -------------------------------------------------------------------------------------
# Author : Steve Purkis <spurkis@mkdoc.com>
# Copyright : (c) 2003, MKDoc Holdings Ltd.
# -------------------------------------------------------------------------------------
package flo::plugin::Account::Remove;
use flo::Record::Ticket;
use flo::Record::Editor;
use flo::Standard;

use Carp;
use Petal;

use MKDoc::XML::Mail;
use MKDoc::Config;
use MKDoc::Mailer;
use MKDoc::Error;

use strict;
use 5.008_000;
use utf8;

use constant ACTION => 'remove';
use constant DEFAULT_EXPIRY_TIME => 7*24*60*60;

use base qw /flo::Plugin/;

sub template_path { '/account/remove' }


##
# $self->http_get;
# ----------------
# Displays the form which lets the editor choose which audiences
# the current document relates to.
##
sub http_get
{
    my $self = shift;
    $self->_create_ticket();
    $self->_send_mail();
    $self->render_http (
	self       => $self,
	object     => $self->parent,
	__input__  => 'XML',
	__output__ => 'XHTML',
       );    
    return 'TERMINATE';
}


##
# $self->_create_ticket;
# ----------------------
# Creates the ticket and inserts it in the database.
# Returns the newly created ticket.
##
sub _create_ticket
{
    my $self  = shift;
    my $email = $self->user()->email();
    
    # clear old tickets & remove previous tickets
    # this is so that a user can re-ask for a ticket
    # rather than wait for his ticket to have expired
    flo::Record::Ticket::clear_old_tickets_from_db();
    my $ticket_table = flo::Standard::table ('Tickets');
    $ticket_table->delete ( Email => $email, Action => ACTION );
    
    # create a new ticket and returns it
    my $ticket = new flo::Record::Ticket;
    eval {
	$ticket->generateId;
	$ticket->Email ($email);
	$ticket->ExpiryDate (time + DEFAULT_EXPIRY_TIME);
	$ticket->Action ( ACTION );
	$ticket->saveToDb();
    };
    
    defined $@ and $@ and do {
	warn $@;
	new MKDoc::Ouch 'mail/cannot_create';
    };
    
    $self->{ticket} = $ticket;
    return $ticket;
}


##
# $self->_send_mail ($ticket);
# ----------------------------
# Sends the email...
##
sub _send_mail
{
    my $self   = shift;
    my $ticket = shift || $self->{ticket};
    
    my $config = new MKDoc::Config ('flo.plugin.Account');
    local *flo::plugin::Account::Remove::template_path;
    *flo::plugin::Account::Remove::template_path = sub { '/account/emails/remove' };
    
    my $xml_mail = eval {
	$self->render (
	    self       => $self,
	    object     => $self->parent(),
	    __input__  => 'XML',
	    __output__ => 'XML',
	   );
    };
    defined $@ and $@ and do {
	warn $@;
	new MKDoc::Ouch 'mail/cannot_create';
	return 0;
    };

    eval {
	my $txt_mail = MKDoc::XML::Mail->process ($xml_mail);
	my $mail = new MKDoc::Mailer ($txt_mail);
	$mail->send();
    };
    defined $@ and $@ and do {
	warn $@;
	new MKDoc::Ouch 'mail/cannot_send';
	return 0;
    };
}


1;


__END__
